from qa_data_manager.bind_clinic_user import BindClinicUser
from qa_data_manager.bind_doctor_patient import BindDoctorPatient
from qa_data_manager.bind_doctor_specialties import BindDoctorSpecialties
from qa_data_manager.clinic import GenerateClinic
from qa_data_manager.consultation import GenerateConsultation
from qa_data_manager.patient_problem import GeneratePatientProblem
from qa_data_manager.patient_problem_discomfort import GeneratePatientProblemDiscomfort
from qa_data_manager.patient_problem_slot_patient import GeneratePatientProblemSlotPatient
from qa_data_manager.service import GenerateService
from qa_data_manager.slot_doctor_patient import GenerateSlotDoctorPatient
from qa_data_manager.tracks import GenerateTracks
from qa_data_manager.user import GenerateUser


class ConsultationGenerate:

    def completed_offline(self, slots_ids, patient, tariff_offline, doctor, slot_info):
        # slots_ids - список нарезанных слотов доктора
        # slot_info - строка информации по слоту из таблицы Slot_doctor_table
        # Генерация завершенной консультации между доктором и пациентом
        slot_doc_pat = GenerateSlotDoctorPatient(slots_ids[0], patient,
                                                 tariff_offline).with_patient_source().with_offline_type().with_completed_status().generate()
        consultation = GenerateConsultation(doctor, patient, tariff_offline, slot_info,
                                            slot_doc_pat.get(
                                                'id')).with_type_offline().with_status_completed().generate()
        # Генерация проблемы
        problem = GeneratePatientProblem(patient).generate()
        GenerateTracks(problem).with_status_second_opinion().generate()
        # Генерация уровня дискомфорта к консультации
        GeneratePatientProblemDiscomfort(consultation, problem).generate()
        # Генерация связи между проблемой и консультацией
        GeneratePatientProblemSlotPatient(problem, tariff_offline, slot_doc_pat).generate()

    def completed_online(self, slots_ids, patient, tariff_video, doctor, slot_info):
        # slots_ids - список нарезанных слотов доктора
        # slot_info - строка информации по слоту из таблицы Slot_doctor_table
        # Генерация завершенной консультации между доктором и пациентом
        slot_doc_pat = GenerateSlotDoctorPatient(slots_ids[0], patient,
                                                 tariff_video).with_patient_source().with_online_type().with_completed_status().generate()
        consultation = GenerateConsultation(doctor, patient, tariff_video, slot_info,
                                            slot_doc_pat.get('id')).with_status_completed().generate()
        # Генерация проблемы
        problem = GeneratePatientProblem(patient).generate()
        GenerateTracks(problem).with_status_second_opinion().generate()
        # Генерация уровня дискомфорта к консультации
        GeneratePatientProblemDiscomfort(consultation, problem).generate()
        # Генерация связи между проблемой и консультацией
        GeneratePatientProblemSlotPatient(problem, tariff_video, slot_doc_pat).generate()

    # Генерация записи на слот для проверки ошибочной записи на занятый слот
    def record_on_slot(self, slot_id, patient, tariff):
        GenerateSlotDoctorPatient(slot_id, patient, tariff).with_online_type().generate()


class PreconditionGenerate:
    # Генерация второй клиники и связи с ней, доктора и связи с ним.
    def second_clinic_and_doctor(self, patient):
        # Генерация новой клиники
        new_clinic = GenerateClinic().generate()
        # Генерация связи пациента и новой клиники
        BindClinicUser(patient).with_clinic(new_clinic.get("id")).generate()
        # Генерация доктора
        doctor2 = GenerateUser().doctor().is_activated().with_clinic(
            new_clinic).with_self_appointment().with_primary_acceptance().generate()
        # Генерация специализации доктора
        BindDoctorSpecialties(doctor2).generate()
        # Генерация тарифа для видео
        GenerateService(doctor2).is_active().with_price(10).is_default().generate()
        # Генерация связи между доктором и пациентом
        BindDoctorPatient(doctor2, patient).generate()
        return new_clinic

    def gen_3_tariff(self, gen_doctor):
        # Генерация двух тарифов для чата
        tariff_chat = GenerateService(gen_doctor).with_chat_service_type().is_active().with_price(
            0).is_default().with_duration(5).generate()
        tariff_chat2 = GenerateService(gen_doctor).with_chat_service_type().is_active().with_price(10).with_duration(
            10).generate()
        # Генерация списка "Названия тарифов для чата"
        chat_names_list = []
        chat_names_list.append(tariff_chat.get("name"))
        chat_names_list.append(tariff_chat2.get("name"))
        # Генерация списка "Описания тарифов для чата"
        chat_desc_list = []
        chat_desc_list.append(tariff_chat.get("description"))
        chat_desc_list.append(tariff_chat2.get("description"))
        # Генерация двух тарифов для видеоконсультации
        tariff_video = GenerateService(gen_doctor).is_active().with_price(0).is_default().with_duration(5).generate()
        tariff_video2 = GenerateService(gen_doctor).is_active().with_price(10).with_duration(10).generate()
        # Генерация списка "Названия тарифов для видеоконсультации"
        video_names_list = []
        video_names_list.append(tariff_video.get("name"))
        video_names_list.append(tariff_video2.get("name"))
        # Генерация списка "Описания тарифов для видеоконсультации"
        video_desc_list = []
        video_desc_list.append(tariff_video.get("description"))
        video_desc_list.append(tariff_video2.get("description"))
        # Генерация двух тарифов для приема в клинике
        tariff_offline = GenerateService(gen_doctor).with_offline_service_type().is_active().with_price(
            0).is_default().with_duration(10).with_duration(5).generate()
        tariff_offline2 = GenerateService(gen_doctor).with_offline_service_type().is_active().with_price(
            10).with_duration(
            10).generate()
        # Генерация списка "Названия тарифов для приема в клинике"
        offline_names_list = []
        offline_names_list.append(tariff_offline.get("name"))
        offline_names_list.append(tariff_offline2.get("name"))
        # Генерация списка "Описания тарифов для приема в клинике"
        offline_desc_list = []
        offline_desc_list.append(tariff_offline.get("description"))
        offline_desc_list.append(tariff_offline2.get("description"))
        # Создания списка продолжительностей тарифов
        duration_list = ["Длительность 5 мин", "Длительность 10 мин"]
        # Создания списка стоимости тарифов
        price_list = ["0 ₽", "10 ₽"]
        info_dict = {'chat_names_list': chat_names_list, 'chat_desc_list': chat_desc_list,
                     'video_names_list': video_names_list, 'video_desc_list': video_desc_list,
                     'offline_names_list': offline_names_list, 'offline_desc_list': offline_desc_list,
                     'tariff_video': tariff_video, 'duration_list': duration_list, 'price_list': price_list}
        return info_dict



import datetime

from faker import Faker

from qa_data_manager.data_base_model import SlotsDoctors

"""
Генерация слотов.
    Поля:
        * type - Тип слота; Принимает значения both/online/offlien (Онлайн и оффлнай / Онлайн / Оффлайн); По умолчанию both.
        * doctor_id - Идентификатор доктора; Обязательное поле, устанавливается при инициализации класса из параметра doctor.
        * start_at - Дата и время начала слота; Принимает значеиние datetime (Дата и время); По умолчанию текущее время и дата.
        * duration - Длительность слота; Принимает int значение; По умолчанию 5.
        * end_at - Дата и время завершения слота; По умолчанию высчитывается из полей start_at + duration
        * context - Тип обработки слота; Принимает значения default/admin/doctor/integration 
            (По умолванию / Подтверждения администратором / Подтверждение доктором / Интеграция); По умолчанию default.
        * deleted_at - Дата удаления слота; Принимает значения datetime/None (Дата и время / Null); По умолчанию None.
        * count - Количество слотов для генерации; Принимает int значение; По умолчанию 1.
    
    Методы:
        * with_admin_context() - Устанавливает в поле context значение admin.
        * with_doctor_context() - Устанавливает в поле context  значение doctor.
        * with_integration_context() - Устанавливает в поле context значение integration.
        * with_count():
            - Принимает параметр count.
            - Устанавиввает в поле count значение из параметра count.
        * is_deleted() - Устанавливает в поле deleted_at значение datetime (Текущее время и дата).
        * with_online_type() - Устанавливает в поле type значение online.
        * with_offline_type() - Устанавливает в поле type значение offline.
        * with_duration() :
            - Принимает параметр duration.
            - Устанавливает в поле duration значение из параметра duration.
        * generate():
            - Сохраняет объект GenerateSlotDoctor в базу данных.
            - Возвращает список идентификаторов сгенерированных слотов.
"""


class GenerateSlotDoctor:
    __fake = Faker('ru_RU')
    __time_mask = '%Y-%m-%d %H:%M:%S'

    def __init__(self, doctor):
        self._type = 'both'
        self._doctor_id = doctor.get('id')
        self._start_at = datetime.datetime.now()
        self._duration = 5
        self._end_at = self._start_at + datetime.timedelta(minutes=int(self._duration))
        self._context = 'default'
        self._deleted_at = None
        self._count = 1

    def with_admin_context(self):
        self._context = 'admin'
        return self

    def with_doctor_context(self):
        self._context = 'doctor'
        return self

    def with_integration_context(self):
        self._context = 'integration'
        return self

    def with_count(self, count):
        self._count = count
        return self

    def is_deleted(self):
        self._deleted_at = datetime.datetime.now()
        return self

    def with_online_type(self):
        self._type = 'online'
        return self

    def with_offline_type(self):
        self._type = 'offline'
        return self

    def with_duration(self, duration):
        self._duration = duration
        self._end_at = self._start_at + datetime.timedelta(minutes=int(self._duration))
        return self

    def generate(self):
        i = 1
        list_slot_ids = []
        while (i <= self._count):
            slot_doctor = SlotsDoctors(doctor_id=self._doctor_id,
                                       type=self._type,
                                       start_at=self._start_at,
                                       duration=self._duration,
                                       end_at=self._end_at,
                                       context=self._context,
                                       created_at=datetime.datetime.now(),
                                       updated_at=datetime.datetime.now())
            slot_doctor.save()
            list_slot_ids.append(slot_doctor.id)
            i = i + 1
            self._start_at = self._end_at
            self._end_at = self._start_at + datetime.timedelta(minutes=int(self._duration))

        return list_slot_ids



import datetime

from playhouse.shortcuts import model_to_dict

from qa_data_manager.data_base_model import RecommendedConsultations


class GenerateRecommendedConsultations:
    def __init__(self, doctor, patient):
        self._date_time = datetime.datetime.now() + datetime.timedelta(hours=int('24'))
        self._deleted_at = None
        self._doctor = doctor.get('id')
        self._is_ignored = 0
        self._is_reminded = None
        self._patient = patient.get('id')
        self._previous_slot_patient = None
        self._slot_patient = None

    def with_previous_slot(self, slot_doctor_patient):
        self._previous_slot_patient = slot_doctor_patient.get('id')
        return self

    def with_recommended_time(self, time):
        self._date_time = datetime.datetime.now() + datetime.timedelta(hours=int(time))
        return self

    def with_deleted_at(self):
        self._deleted_at = datetime.datetime.now()
        return self

    def generate(self):
        recommended_consultation = RecommendedConsultations(created_at=datetime.datetime.now(),
                                                            date_time=self._date_time,
                                                            deleted_at=self._deleted_at,
                                                            doctor=self._doctor,
                                                            is_ignored=self._is_ignored,
                                                            is_reminded=self._is_reminded,
                                                            patient=self._patient,
                                                            previous_slot_patient=self._previous_slot_patient,
                                                            slot_patient=self._slot_patient,
                                                            updated_at=datetime.datetime.now())

        recommended_consultation.save()
        return model_to_dict(recommended_consultation)



from datetime import datetime

from qa_data_manager.data_base_model import SlotsDoctors, ConfirmationCodes, Users, DoctorPatient, Chats, ClinicUser, \
    Clinics, Services


class SlotDoctorTable:
    # Вся строка из таблицы по id слота
    def slot_info_by_id(self, slot_id):
        return SlotsDoctors.select().where(SlotsDoctors.id == slot_id).get()

    # Дата и время начала определенного слота по id
    def start_at_by_id(self, slot_id):
        return datetime.strptime(str(self.slot_info_by_id(slot_id).start_at), "%Y-%m-%d %H:%M:%S")

    # Дата и время начала определенного слота по id
    def slot_info_by_doctor(self, doctor):
        return SlotsDoctors.select().where(SlotsDoctors.doctor == doctor.get("id"))

    # Дата и время начала определенного слота по id
    def first_slot_info_by_list_of_slots_ids(self, slots_ids):
        return SlotsDoctors.select().where(SlotsDoctors.id == slots_ids[0]).get()

    # Дата и время всех слотов на определенный день
    def list_of_slots_start_at_by_day_and_gen_person(self, day, gen_doctor):
        slots_list_bd = []
        slots_bd = SlotsDoctors.select().where(
            (SlotsDoctors.start_at.contains(f"{day}")) & (SlotsDoctors.doctor == gen_doctor.get("id")))
        for s in slots_bd.objects():
            slots_list_bd.append(datetime.strftime((s.start_at), "%H:%M"))
        return slots_list_bd

    # Дата и время всех слотов на определенный день
    def list_of_slots_start_at_by_gen_person(self, gen_doctor):
        slots_list_bd = []
        slots_bd = self.slot_info_by_doctor(gen_doctor)
        for s in slots_bd.objects():
            slots_list_bd.append(datetime.strftime((s.start_at), "%H:%M"))
        return slots_list_bd


class UserTable:
    # Вся строка из таблицы по сгенерированному пользователю
    def info_by_gen_person(self, person):
        return Users.select().where(Users.phone == person.get('phone')).get()

    # Код по сгенерированному пользователю
    def code_by_gen_person(self, person):
        return self.info_by_gen_person(person).code

    # Registration code по сгенерированному пользователю
    def registration_code_by_gen_person(self, person):
        return self.info_by_gen_person(person).registration_code

    # Ссылка на аватарку по сгенерированному пользователю
    def avatar_by_gen_person(self, person):
        return self.info_by_gen_person(person).avatar

    # Кол-во пациентов по номеру телефона (маска + сам номер)
    def count_of_patient_by_mask_and_number(self, mask, patient_phone):
        return Users.select().where((Users.phone == f"{mask}{patient_phone}") & (Users.type == "patient") & (
            Users.deleted_at.is_null())).count()


class ConfirmationCodesTable:
    # Код по пользователю через запрос к БД
    def get_code_by_req_person(self, person):
        return ConfirmationCodes.select().where(ConfirmationCodes.user_id == person.id).get().code

    # Код по сгенерированному пользователю
    def get_code_by_gen_person(self, person):
        return ConfirmationCodes.select().where(ConfirmationCodes.user_id == person.get("id")).get().code


class DoctorPatientTable:
    # Кол-во записей между id двух пользователей через сгенерированного доктора и запрос к БД по пациенту
    def count_of_value_by_gen_doc_req_pat(self, doctor, patient):
        return DoctorPatient.select().where(
            (DoctorPatient.doctor == doctor.get('id')) & (DoctorPatient.patient == patient.id)).count()

    # Кол-во записей между id двух пользователей через сгенерированного доктора и сгенерированного пациента
    def count_of_value_by_gen_doc_gen_pat(self, doctor, patient):
        return DoctorPatient.select().where(
            (DoctorPatient.doctor == doctor.get('id')) & (DoctorPatient.patient == patient.get('id'))).count()


class ChatsTable:
    # Кол-во записей между id двух пользователей через сгенерированного доктора и запрос к БД по пациенту
    def count_of_value_by_gen_doc_req_pat(self, doctor, patient):
        return Chats.select().where(
            (Chats.doctor == doctor.get('id')) & (Chats.patient == patient.id)).count()

    # Кол-во записей между id двух пользователей через сгенерированного доктора и сгенерированного пациента
    def count_of_value_by_gen_doc_gen_pat(self, doctor, patient):
        return Chats.select().where(
            (Chats.doctor == doctor.get('id')) & (Chats.patient == patient.get('id'))).count()


class ClinicUserTable:
    # id клиники по сгенерированному пользователю
    def clinic_id_by_gen_person(self, person):
        return ClinicUser.select().where(ClinicUser.user == person.get("id")).get().clinic


class ClinicTable:
    # Название клиники по id клиники
    def name_by_clinic_id(self, clinic_id):
        return Clinics.select().where(Clinics.id == clinic_id).get().name

    # Название клиники по сгенерированной клинике
    def name_by_gen_clinic(self, gen_clinic):
        return Clinics.select().where(Clinics.id == gen_clinic.get("id")).get().name

    # Город и улица клиники по id клиники
    def address_by_clinic_id(self, clinic_id):
        info = Clinics.select().where(Clinics.id == clinic_id).get()
        return info.city + ", " + info.street

    # Город и улица клиники по сгенерированной клинике
    def address_by_gen_clinic(self, gen_clinic):
        info = Clinics.select().where(Clinics.id == gen_clinic.get("id")).get()
        return info.city + ", " + info.street


class ServicesTable:
    # Деактивация тарифа
    def deactivate_tariff(self, tariff):
        Services.update(active=0).where(Services.id == tariff.get("id")).execute()



from playhouse.shortcuts import model_to_dict

from qa_data_manager.data_base_model import Family


class BindFamilyAccounts:

    def __init__(self, main_acc, child_acc):
        self._child_account = child_acc.get('id')
        self._child_role = 'child'
        self._main_account = main_acc.get('id')

    def with_child_role(self):
        self._child_role = 'child'
        return self

    def with_spouse_role(self):
        self._child_role = 'spouse'
        return self

    def generate(self):
        family = Family(child_account=self._child_account,
                        child_role=self._child_role,
                        main_account=self._main_account)

        family.save()
        return model_to_dict(family)



import datetime

from playhouse.shortcuts import model_to_dict

from qa_data_manager.data_base_model import PatientProblemSlotPatient


class GeneratePatientProblemSlotPatient:

    def __init__(self, problem, serivce, slot_patient):
        self._patient_problem = problem.get('id')
        self._service_id = serivce.get('id')
        self._slot_patient = slot_patient.get('id')

    def generate(self):
        problem_slot_patient = PatientProblemSlotPatient(created_at=datetime.datetime.now(),
                                                         patient_problem=self._patient_problem,
                                                         service_id=self._service_id,
                                                         slot_patient=self._slot_patient,
                                                         updated_at=datetime.datetime.now())

        problem_slot_patient.save()
        return model_to_dict(problem_slot_patient)



import datetime

from playhouse.shortcuts import model_to_dict

from qa_data_manager.data_base_model import Conclusions


class GenerateConclusions:

    def __init__(self, appoint, serv_pat):
        self._appointment = appoint.get('id')
        self._created_at = datetime.datetime.now()
        self._service_patient = serv_pat.get('id')
        self._text = 'text for test'
        self._updated_at = self._created_at

    def with_text(self, my_text):
        self._text = my_text
        return self

    def generate(self):
        conclusion = Conclusions(appointment=self._appointment,
                                 created_at=datetime.datetime.now(),
                                 service_patient=self._service_patient,
                                 text=self._text,
                                 updated_at=datetime.datetime.now())

        conclusion.save()
        return model_to_dict(conclusion)



import datetime

from playhouse.shortcuts import model_to_dict

from qa_data_manager.data_base_model import Tracks


class GenerateTracks:

    def __init__(self, patient_problem):
        self._entity_id = patient_problem.get('id')
        self._entity_type = 'patient_problem'
        self._flow = None
        self._status = 'auto'

    def with_entity_type_slot_doctor(self):
        self._entity_type = 'slot_doctor'
        return self

    def with_status_appointed_offline_reception(self):
        self._status = 'appointed_offline_reception'
        return self

    def with_status_approved(self):
        self._status = 'approved'
        return self

    def with_status_canceled(self):
        self._status = 'canceled'
        return self

    def with_status_closed(self):
        self._status = 'closed'
        return self

    def with_status_diagnostics_assigned(self):
        self._status = 'diagnostics_assigned'
        return self

    def with_status_primary_appointment(self):
        self._status = 'primary_appointment'
        return self

    def with_status_pending(self):
        self._status = 'pending'
        return self

    def with_status_repeated_appointment(self):
        self._status = 'repeated_appointment'
        return self

    def with_status_second_opinion(self):
        self._status = 'second_opinion'
        return self

    def with_status_transcript_of_analyzes(self):
        self._status = 'transcript_of_analyzes'
        return self

    def with_status_treatment_completed(self):
        self._status = 'treatment_completed'
        return self

    def with_status_treatment_prescribed(self):
        self._status = 'treatment_prescribed'
        return self

    def with_status_wait_payment(self):
        self._status = 'wait_payment'
        return self

    def generate(self):
        tracks = Tracks(created_at=datetime.datetime.now(),
                        entity_id=self._entity_id,
                        entity_type=self._entity_type,
                        flow=self._flow,
                        status=self._status,
                        updated_at=datetime.datetime.now())

        tracks.save()
        return model_to_dict(tracks)
